#!/usr/bin/env python
# -*- coding: utf-8 -*-
from marshmallow import Schema, fields, INCLUDE, EXCLUDE
from datetime import datetime
from flask_templates.views.base import ViewValidationError


class DateTimeEXT(fields.DateTime):
    """
    Class extends marshmallow standart DateTime with "timestamp" format.
    """

    SERIALIZATION_FUNCS = \
        fields.DateTime.SERIALIZATION_FUNCS.copy()
    DESERIALIZATION_FUNCS = \
        fields.DateTime.DESERIALIZATION_FUNCS.copy()

    SERIALIZATION_FUNCS['timestamp'] = lambda x: x.timestamp()
    DESERIALIZATION_FUNCS['timestamp'] = lambda x: datetime.fromtimestamp(float(x))


class ViewBaseSchema(Schema):
    def handle_error(self, exc, data, **kwargs):
        raise ViewValidationError(str(exc.messages), data)

    def validate_s(self,
                      data,
                      *,
                      many: bool = None,
                      partial):
        self._do_load(data, many=many, partial=partial, postprocess=False)


class ReqSchema(Schema):
    token = fields.Str(required=True)

    def handle_error(self, exc, data, **kwargs):
        raise ViewValidationError(str(exc.messages), data)






from datetime import datetime as dt, timedelta
from functools import wraps

DATETIME_FORMAT = '%Y-%m-%d %H:%M:%S'


class Keeper:

    def fetch(self, key: str):
        pass

    def set_status(self, key: str, status: int):
        pass

    def keep(self, key: str, time_value: dt, value, status=0):
        pass


class DefaultKeeper(Keeper):
    collection = dict()

    def fetch(self, key: str):
        return self.collection.get(key)

    def set_status(self, key: str, status: int):
        value = self.collection.get(key)
        if value:
            value['status'] = status

    def keep(self, key: str, time_value: dt, value, status=0):
        self.collection[key] = {"key": key, "time_value": dt.strftime(time_value, DATETIME_FORMAT),
                                "value": str(value) if value else '',
                                'status': status}


def fresher(key, default_value=None, default_time_value=None, keeper: Keeper = DefaultKeeper, skip=0):
    def wrapper(func):
        @wraps(func)
        def wrappers(*args, **kwargs):
            # 当前时间
            now_dt = dt.now()
            skiper = skip
            # 获取原值
            key_info = keeper().fetch(key=key)
            if key_info:
                skiper = key_info['status']
                src_value = key_info['value']
                src_time_value = dt.strptime(key_info['time_value'], DATETIME_FORMAT)
            else:
                src_value = default_value if default_value else None
                src_time_value = default_time_value if default_time_value else now_dt - timedelta(days=30)
            if skiper:
                return func(*args, **kwargs)
            res = func(*args, _src_value=src_value, _src_time=src_time_value, _now_time=now_dt, **kwargs)
            keeper().keep(key, now_dt, res, status=0)
            return res

        return wrappers

    return wrapper


if __name__ == '__main__':
    import time


    @fresher("a")
    def test(_src_value=None, _src_time=None, _now_time=None):
        print(_src_time, _src_value, _now_time)


    while True:
        test()
        time.sleep(10)



from unittest import TestCase
from flask_templates.common.class_holder import class_register, ClassHolder, class_getter


def a(self):
    return 123


class A:
    pass


@class_register()
class B(A):
    def __init__(self):
        pass


@class_register()
class C:

    @property
    @class_getter(A)
    def cc(self):
        pass

    def __init__(self):
        self.c = "123"


class TestClassHolder(TestCase):

    def test_put(self):
        c = ClassHolder().get(C)
        print(c)
        c2 = C()
        print(c2)
        c3 = ClassHolder().get(C)
        print(c3)


from unittest import TestCase


class TestExample(TestCase):
    def test_sum(self):
        from flask_templates.async_task import async_tasker

        while True:
            res = async_tasker.delay('flask_templates.async_task.example.sum', 1, 3)
            print(res.state)
            try:
                print(res.get(timeout=10))

            except Exception as e:
                import traceback
                traceback.print_exc()




# -*- coding: utf-8 -*-
"""
------------------------------------------------------------------
    File Name:              example
    Description:            
    Author:                 hzy
    date:                   2019/8/24
------------------------------------------------------------------
    Change Activity:
                            2019/8/24
------------------------------------------------------------------
"""
from flask_templates.common.logger import logger

__author__ = 'hzy'

import time
from flask_templates.views import api
from flask_templates.views.base import check_request_params
from flask_templates.service.example import index


@api.route('/')
@check_request_params(auth=True)
def example_index(**kwargs):
    """
/**
* showdoc
* @catalog 测试文档/用户相关
* @title 用户登录
* @description 用户登录的接口
* @method get
* @url https://www.showdoc.cc/home/user/login
* @header token 可选 string 设备token
* @param username 必选 string 用户名
* @param password 必选 string 密码
* @param name 可选 string 用户昵称
* @return {"error_code":0,"data":{"uid":"1","username":"12154545","name":"吴系挂","groupid":2,"reg_time":"1436864169","last_login_time":"0"}}
* @return_param groupid int 用户组id
* @return_param name string 用户昵称
* @remark 这里是备注信息
* @number 99
*/
    Parameters
    ----------
    kwargs

    Returns
    -------

    """
    logger.error('333')
    return index(**kwargs)



if __name__ == "__main__":
    start_t = time.time()

    pass

    print("use time: %s" % (time.time() - start_t))



# -*- coding: utf-8 -*-
"""
----------------------------------------------------
    File Name:              cache
    Description             
    Author:                 develop
    date:                   2019/4/8
----------------------------------------------------
"""
__author__ = 'develop'

import time
import simplejson as json
import pickle
# import json

from flask_templates.common.redis_module import MyRedis, cache_decorator


def __combine_key(src_id, cache_type=None):
    key = src_id
    if cache_type is None:
        key = str(key)
    else:
        key = str(cache_type) + "::" + str(key)
    return key


@cache_decorator()
def set_json_cache(src_id, content, cache_type=None, ex=None, redis_client=None, **kwargs):
    key = __combine_key(src_id, cache_type)
    res = redis_client.set(key, json.dumps(content))
    if ex:
        redis_client.expire(key, ex)
    return res


@cache_decorator(master=False)
def get_json_cache(src_id, cache_type=None, redis_client=None, **kwargs):
    key = __combine_key(src_id, cache_type)
    res = redis_client.get(key)
    if res is None:
        return None
    return json.loads(res)


@cache_decorator()
def set_pickle_cache(src_id, content, cache_type=None, redis_client=None, **kwargs):
    key = __combine_key(src_id, cache_type)
    value = pickle.dumps(content)
    return redis_client.set(key, value)


@cache_decorator(master=False)
def get_pickle_cache(src_id, cache_type=None, redis_client=None, **kwargs):
    key = __combine_key(src_id, cache_type)
    res = redis_client.get(key)
    if res is None:
        return None
    return pickle.loads(res)


@cache_decorator()
def clear_cache(src_id, cache_type=None, redis_client=None, **kwargs):
    key = __combine_key(src_id, cache_type)
    return redis_client.delete(key)


@cache_decorator(master=False)
def keys_cache(pattern, redis_client=None, **kwargs):
    return redis_client.keys(pattern)


@cache_decorator()
def set_hash_cache(src_id, content, cache_type=None, redis_client=None, **kwargs):
    """
    设置缓存
    :param src_id:
    :param content:
    :param cache_type:
    :param redis_client:
    :return:
    """
    key = __combine_key(src_id, cache_type)
    return redis_client.hmset(key, content)


@cache_decorator(master=False)
def get_hash_cache(src_id, cache_type=None, redis_client=None, **kwargs):
    key = __combine_key(src_id, cache_type)
    return redis_client.hgetall(key)


@cache_decorator()
def clear_db(redis_client=None, **kwargs):
    return redis_client.flushdb()


@cache_decorator()
def set_hash_item_cache(src_id, content_key, content_value, cache_type=None, redis_client=None, **kwargs):
    key = __combine_key(src_id, cache_type)
    res = redis_client.hset(key, content_key, content_value)
    return res


@cache_decorator(master=False)
def get_hash_item_cache(src_id, content_key, cache_type=None, redis_client=None, **kwargs):
    key = __combine_key(src_id, cache_type)
    return redis_client.hget(key, content_key)


@cache_decorator()
def clear_hash_item_cache(src_id, *content_keys, cache_type=None, redis_client=None, **kwargs):
    key = __combine_key(src_id, cache_type)
    return redis_client.hdel(key, *content_keys)


@cache_decorator()
def set_cache(src_id, content_value, cache_type=None, redis_client=None, **kwargs):
    key = __combine_key(src_id, cache_type)
    return redis_client.set(key, content_value)


@cache_decorator(master=False)
def get_cache(src_id, cache_type=None, redis_client=None, **kwargs):
    key = __combine_key(src_id, cache_type)
    return redis_client.get(key)


@cache_decorator()
def set_list_cache(src_id, content_values, cache_type=None, redis_client=None, **kwargs):
    key = __combine_key(src_id, cache_type)
    return redis_client.lpush(key, *content_values)


@cache_decorator(master=False)
def get_list_cache(src_id, start=0, end=None, cache_type=None, redis_client=None, **kwargs):
    key = __combine_key(src_id, cache_type)
    if not end:
        end = -1
    return redis_client.lrange(key, start, end)


if __name__ == "__main__":
    set_json_cache("a", 1, None, None, redis_name="default")
    print(get_json_cache("a", None, MyRedis().get_redis_client()))



from celery import Celery
from flask_templates.configs import config_update, BEFORE_CONFIG, template
from importlib import import_module
"""
run 'celery -A flask_templates.async_task.cele worker -l INFO  -P gevent  -c 10' in windows
monitor 'celery flower --broker=redis://r2.dsjcj.cc:10168/2'
"""

if template.CELERY_CONFIG.pop("enable",False):
    cele = Celery(__name__, broker=template.CELERY_CONFIG['broker_url'])
    cele.conf.update(template.CELERY_CONFIG)
else:
    cele = Celery()

@cele.task
def async_tasker(func,*args,**kwargs):
    module,func = func.rsplit(".", maxsplit=1)
    func_m = import_module(module)
    return getattr(func_m,func)(*args,**kwargs)

if __name__ == '__main__':
    pass


import threading
import sys
import functools
from inspect import isfunction
from copy import deepcopy


class ClassHolder(object):
    _instance_lock = threading.Lock()
    __class_dict = dict()
    __func_to_class = dict()

    def __init__(self):
        pass

    def __new__(cls, *args, **kwargs):
        if not hasattr(ClassHolder, "_instance"):
            with ClassHolder._instance_lock:
                if not hasattr(ClassHolder, "_instance"):
                    ClassHolder._instance = object.__new__(cls)
        return ClassHolder._instance

    def get_class_dict(self):
        return deepcopy(self.__class_dict)

    def put(self, obj, special_key='__default'):
        # class_module = obj.__class__.__module__
        # class_name = obj.__class__.__name__
        # obj_class = getattr(sys.modules[class_module], class_name)
        with ClassHolder._instance_lock:
            obj._fixed = True
            self.__class_dict[obj.__class__] = {"obj": obj, "name": special_key}

    def get(self, clazz, special_key='__default', is_singleton=True):
        current_result = []
        if isfunction(clazz):
            clazz = self.get_fix(clazz)
        for one_class, value in self.__class_dict.items():
            if (one_class == clazz or issubclass(one_class, clazz)) and value['name'] == special_key:
                if current_result:
                    raise Exception("class {} conflict with {}".format(current_result[0][0], one_class))
                current_result.append((one_class, value))
        if not current_result:
            raise Exception("none of {} can be found".format(clazz))

        exist = current_result[0][1]['obj']
        if is_singleton:
            return exist
        return deepcopy(exist)

    def fix(self, func, clazz):
        self.__func_to_class[func] = clazz

    def get_fix(self, func):
        return self.__func_to_class.get(func)


def class_getter(name=None):
    def wrapper(property_get):
        def get(obj):
            if getattr(obj, "_fixed",False):
                return ClassHolder().get(name)
            return name()
        return get
    return wrapper



def class_register(name="__default"):
    """
    正在尝试注入，beta
    :param name:
    :return:
    """
    def wrapper(func):
        # @functools.wraps(func)
        # def request(*args, **kwargs):
        #     filled_obj = func(*args, **kwargs)
        #     ClassHolder().put(filled_obj, special_key=name)
        #     # ClassHolder().put(args[0],special_key=name)
        #     return filled_obj
        #
        # ClassHolder().fix(request, func)
        # return request

        # 创建对象
        ClassHolder().put(func(), special_key=name)
        return func

    return wrapper





# -*- coding: utf-8 -*-
"""
------------------------------------------------------------------
    File Name:              manage
    Description:            
    Author:                 hzy
    date:                   2019/8/23
------------------------------------------------------------------
    Change Activity:
                            2019/8/23
------------------------------------------------------------------
"""
__author__ = 'hzy'

import time
from datetime import datetime

from flask_templates.common.logger import logger
from flask_templates import FlaskTemplate
from flask_templates.configs.template import WEB_CONFIG
from flask_templates.nacosinstance import NacosClient
from flask_templates.common.class_holder import class_register,class_getter,ClassHolder

app = FlaskTemplate().app
conf = WEB_CONFIG


@app.route('/')
def index():
    logger.error('error test')
    return 'hello,world!'

@class_register()
class Register:

    @class_register()
    class RegisterInner(NacosClient):
        pass

    @property
    @class_getter(NacosClient)
    def nacos_client(self):
        pass

    def __init__(self):
        self.nacos_client.start_heartbeat(app)

if __name__ == "__main__":
    start_t = time.time()
    logger.info('^v^ ^V^server start on {}'.format(conf['port']))
    app.run(host=conf['host'], port=conf['port'], debug=True, use_reloader=False)
    pass

    print("use time: %s" % (time.time() - start_t))



from flask_templates.common.utils import module_import, get_host_ip

module_import("nacos", sdk_name='nacos-sdk-python')

import nacos
import datetime
import yaml
from flask_templates.common.requester import ServiceRegisterCenter
from flask_templates.configs import BASE_CONFIG, config_update, config_replace
from flask_templates.configs.template import WEB_CONFIG
from flask_templates.configs.nacos import NACOS_CONFIG
from flask_templates.common.utils import read_yaml
from flask_templates.common.logger import logger
from flask_apscheduler import APScheduler

NAMESPACE = ""

# Set the following values if authentication mode is enabled on the server
USERNAME = None
PASSWORD = None


class NacosClient(ServiceRegisterCenter):

    def __init__(self, app=None, only_reader=False):
        self.app = app
        self.config = NACOS_CONFIG
        self.discovery_config = self.config.get('discovery', {})
        self.config_config = self.config.get('config', {})
        self.only_reader = only_reader or self.discovery_config.get("only_reader")
        self.pyctuator = None

        self.client = nacos.NacosClient(self.server_addr, namespace=NAMESPACE, username=USERNAME, password=PASSWORD)
        if self.config_enable:
            config_update(BASE_CONFIG, setting=self.get_config())
            config_replace(BASE_CONFIG)
        if self.discovery_enable and not self.only_reader:
            self.scheduler = getattr(self.app, "apscheduler") if self.app and getattr(self.app, "apscheduler",
                                                                                      None) else APScheduler()
            self.__register(app=self.app)
        nacos.client.logger = logger

    @property
    def server_addr(self):
        return self.config.get("server-addr")

    @property
    def discovery_enable(self):
        return self.discovery_config.get("enable", False)

    @property
    def config_enable(self):
        return self.config_config.get("enable", False)

    def get_config(self):
        config = {}
        if self.config_enable:
            default_data_id = "{}-{}.{}".format(WEB_CONFIG.get('service_name'), BASE_CONFIG.get("env", "dev"),
                                                self.config_config.get("file-extension", "yaml"))
            default_group = self.config_config.get("group", "default")
            config_str = self.client.get_config(data_id=default_data_id, group=default_group, timeout=None,
                                                no_snapshot=None)
            if config_str:
                config.update(yaml.safe_load(config_str))
            else:
                logger.warn("data-id {} does't has any content".format(default_data_id))
            extension_configs = self.config_config.get("extension-configs", [])
            for item in extension_configs:
                config_str = self.client.get_config(data_id=item['data-id'], group=item.get('group', default_group),
                                                    timeout=None,
                                                    no_snapshot=None)
                if config_str:
                    config.update(yaml.safe_load(config_str))
                else:
                    logger.warn("data-id {} does't has any content".format(item['data-id']))
        return config

    def __register(self, app=None):
        if app and self.discovery_config.get("boot_admin", False):
            module_import("pyctuator")
            from pyctuator.pyctuator import Pyctuator
            self.pyctuator = Pyctuator(
                app,
                WEB_CONFIG.get('service_name'),
                app_url="http://wealthwechat.dsjcj.cc",
                pyctuator_endpoint_url="http://wealthwechat.dsjcj.cc/pyctuator",
                # registration_url="http://wealth.afa.ai:10008/admin/instances"
                registration_url=None
            )
        self.client.add_naming_instance(service_name=WEB_CONFIG.get('service_name'),
                                        ip=WEB_CONFIG.get("service_ip") if WEB_CONFIG.get(
                                            "service_ip") != '0.0.0.0' else get_host_ip(),
                                        port=WEB_CONFIG.get('service_port'), weight=1,
                                        cluster_name="DEFAULT",
                                        metadata={"preserved.register.source": "FLASK",
                                                  "management.context-path": "pyctuator"},
                                        enable=True, healthy=False,
                                        group_name=self.discovery_config.get("group", "default"))

    def nacos_heartbeat_func(self):
        self.client.send_heartbeat(WEB_CONFIG.get('service_name'), ip=WEB_CONFIG.get("service_ip") if WEB_CONFIG.get(
            "service_ip") != '0.0.0.0' else get_host_ip(),
                                   port=WEB_CONFIG.get('service_port'),
                                   weight=1,
                                   cluster_name="DEFAULT",
                                   metadata={"preserved.register.source": "FLASK",
                                             "management.context-path": "pyctuator"},
                                   ephemeral=True, group_name=self.discovery_config.get("group", "default"))
        # print(".")

    def start_heartbeat(self, app=None, seconds=15):
        if self.discovery_enable and not self.only_reader:
            if self.discovery_config.get("boot_admin", False) and not self.pyctuator:
                module_import("pyctuator")
                from pyctuator.pyctuator import Pyctuator
                self.pyctuator = Pyctuator(
                    app,
                    WEB_CONFIG.get('service_name'),
                    app_url="http://wealthwechat.dsjcj.cc",
                    pyctuator_endpoint_url="http://wealthwechat.dsjcj.cc/pyctuator",
                    # registration_url="http://wealth.afa.ai:10008/admin/instances"
                    registration_url=None
                )
            self.scheduler.add_job(func=self.nacos_heartbeat_func, id='nacos_heartbeat', trigger='interval',
                                   seconds=seconds,
                                   replace_existing=True, start_date=datetime.datetime.now())
            if not self.scheduler.running:
                app.config['SCHEDULER_API_ENABLED'] = True
                self.scheduler.init_app(app=app if app else self.app)
                self.scheduler.start()

    def stop_heartbeat(self):
        if self.discovery_enable and not self.only_reader:
            self.scheduler.remove_all_jobs()
            self.scheduler.shutdown()

    def get_servers(self, service_name):
        return [(x['ip'], x['port']) for x in
                self.client.list_naming_instance(service_name,
                                                 group_name=self.discovery_config.get("group", "default"),
                                                 healthy_only=True)['hosts']]


if __name__ == '__main__':
    import time
    from flask_templates import FlaskTemplate

    client = NacosClient(FlaskTemplate().app)
    client.start_heartbeat()
    print(client.get_servers(WEB_CONFIG.get('service_name')))
    while True:
        time.sleep(1)



import time
import os
import re
from flask_templates.common.utils import read_yaml, iter_change, mix_change,replace_flat_dict,flat_collect
from collections import OrderedDict

# 基础配置文件名
before_config_filename = 'bootstrap'
base_config_filename = 'application'
# 基础配置路径
init_resource_path = os.path.abspath(
    os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), '..', 'flask_templates', 'resource'))
# 初始配置文件路径
before_env_path = os.path.abspath(os.path.join(init_resource_path, before_config_filename + '.yaml'))
current_env_path = os.path.abspath(os.path.join(init_resource_path, base_config_filename + '.yaml'))
# 基础配置
BEFORE_CONFIG = read_yaml(before_env_path)
BASE_CONFIG = read_yaml(current_env_path)
FINAL_SETTING = {}


def config_update(src_config=BASE_CONFIG, resource_path=None, setting={}, final_setting={}, call_fix=None,
                  src_config_filename=base_config_filename):
    extra_config_dict = dict()
    based_project_path = None
    if resource_path and os.path.exists(resource_path):

        for file in os.listdir(resource_path):
            file_path = os.path.join(resource_path, file)
            if os.path.isdir(file_path):
                continue
            filename = file.rsplit(".yaml", 1)[0]
            if filename == src_config_filename:
                based_project_path = file_path
            elif filename.startswith(src_config_filename + "_"):
                extra_config_dict[filename] = file_path
    # 本地配置文件（对应环境）
    base_project_config = read_yaml(based_project_path)
    mix_change(src_config, base_project_config)
    # 本地配置文件（对应环境）
    env = setting.get("env", src_config.get("env"))
    if env:
        extra_project_config = read_yaml(extra_config_dict.get(src_config_filename + '_' + env))
        mix_change(src_config, extra_project_config)
    # 配置替换,来自服务器的配置可以在这步生效
    mix_change(src_config, setting)
    # 环境变量
    iter_change(src_config, os.environ)
    # 最终配置替换
    FINAL_SETTING.update(final_setting)
    mix_change(src_config, FINAL_SETTING)
    # fix callback
    if call_fix:
        [x() for x in call_fix]
    return src_config


def config_replace(config):
    flat_dict = {}
    flat_collect(config, flat_dict=flat_dict)
    for key in flat_dict:
        replace_flat_dict(key, flat_dict[key]['value'], flat_dict[key]['relative_map'], flat_dict)
    iter_change(config, {k: v['value'] for k, v in flat_dict.items()}, sep=".")


if __name__ == "__main__":
    print(BASE_CONFIG)
    print(config_replace(BASE_CONFIG))


from unittest import TestCase

from flask_templates.common.utils import *

class TestReplace_variables(TestCase):
    def test_replace_variables(self):
        self.assertEqual(replace_variables("adasd${ad:123}dasd,${ad}ffggtgt",ad='ccc'),"adasdcccdasd,cccffggtgt")
        self.assertEqual(replace_variables("adasd${ad:123}dasd,ffggtgt"),"adasd123dasd,ffggtgt")



from ..configs import BASE_CONFIG

# web服务
WEB_CONFIG = BASE_CONFIG.get('web', {})
# template config
TEMPLATE_CONFIG = BASE_CONFIG.get('flask_template', {})
# 日志
LOG_CONFIG = TEMPLATE_CONFIG.get('log', {})
# requester
REQUESTER_CONFIG = TEMPLATE_CONFIG.get('requester', {})
# celery
CELERY_CONFIG = BASE_CONFIG.get('celery', {})
# 数据库
DB_CONFIG = TEMPLATE_CONFIG.get('db', {})
# mysql
MYSQL_CONFIG = DB_CONFIG.get('mysql', {})
# USE_SLAVE_CONFIG = DB_CONFIG.get('use_slave')
# SLAVE_MYSQL_CONFIG = DB_CONFIG.get('slave_mysql')
# redis
REDIS_CONFIG = DB_CONFIG.get('redis', {})


def config_fix():
    global TEMPLATE_CONFIG, LOG_CONFIG, WEB_CONFIG, DB_CONFIG, MYSQL_CONFIG, REDIS_CONFIG, REQUESTER_CONFIG,CELERY_CONFIG
    TEMPLATE_CONFIG = BASE_CONFIG.get('flask_template', {})
    LOG_CONFIG = TEMPLATE_CONFIG.get('log', {})
    REQUESTER_CONFIG = TEMPLATE_CONFIG.get('requester', {})
    DB_CONFIG = TEMPLATE_CONFIG.get('db', {})
    MYSQL_CONFIG = DB_CONFIG.get('mysql', {})
    REDIS_CONFIG = DB_CONFIG.get('redis', {})
    WEB_CONFIG = BASE_CONFIG.get('web', {})
    CELERY_CONFIG = BASE_CONFIG.get('celery', {})



# -*- coding: utf-8 -*-
"""
----------------------------------------------------
    File Name:              __init__.py
    Description
    Author:                 develop
    date:                   2019/4/4
----------------------------------------------------
"""
__author__ = 'develop'

import traceback
from sqlalchemy import or_, and_
from functools import wraps
from sqlalchemy.orm import Query
from flask_templates.common.sqlalchemy_module import MySQL
from flask_templates.model import *


def transaction(master_name=None, session_class_name=None, random_slave=False):
    def wrap(func):
        """
        不与dao里的start_transaction，end_transaction联用
        Parameters
        ----------
        func

        Returns
        -------

        """

        @wraps(func)
        def wrapper(*args, **kwargs):
            if kwargs.get('tr'):
                return func(*args, **kwargs)
            b = TransactionAction(master_name, session_class_name, random_slave)
            b.start_transaction()
            kwargs['tr'] = b
            try:
                res = func(*args, **kwargs)
            except Exception as e:
                try:
                    b.session.rollback()
                    b.session.close()
                except:
                    pass
                raise e
            else:
                b.end_transaction()
            return res

        return wrapper

    return wrap


master_transaction = simple_transaction = transaction(session_class_name=None)
random_slave_transaction = transaction(random_slave=True)
slave_transaction = lambda slave_name: transaction(session_class_name=slave_name)


class BaseDaoError(Exception):

    def __init__(self, msg):
        self.msg = msg

    def __str__(self):
        return self.msg

    def __repr__(self):
        return self.__str__()


class TransactionAction(object):

    def __init__(self, master_name=None, session_class_name=None, random_slave=False):
        self.session_class_name = session_class_name
        self.random_slave = random_slave
        self.session = None
        self.transaction = False
        self.master_name = master_name

    def start_transaction(self):
        if self.transaction:
            raise BaseDaoError('end error!!!transaction has not begun')
        self.start()
        self.transaction = True

    def start(self):
        if self.session:
            return self.session
        self.session = MySQL(master_name=self.master_name).sql_session(self.session_class_name, self.random_slave)
        return self.session

    def end_transaction(self):
        if not self.transaction:
            raise BaseDaoError('end error!!!transaction has not begun')
        self.transaction = False
        self.end()

    def end(self):
        if self.transaction:
            return
        if self.session:
            self.session.commit()
            self.session.close()
            self.session = None


def sql_wrapper(result_func):
    def addition_type(func):
        @wraps(func)
        def wrapper(self, *args, **kwargs):
            temp = func(self, *args, **kwargs)
            if "order_by" in kwargs and kwargs['order_by']:
                temp = temp.order_by(kwargs['order_by'])
            if "desc" in kwargs and kwargs['desc']:
                temp = temp.desc()
            if "limit" in kwargs and kwargs['limit']:
                temp = temp.limit(kwargs['limit'])
            if "offset" in kwargs and kwargs['offset']:
                temp = temp.limit(kwargs['offset'])
            result = result_func(temp)
            self.tr.end()
            return result

        return wrapper

    return addition_type


class BaseDao(object):

    def __init__(self, entry_class, master_name=None, session_class_name=None, rs=False, tr=None):
        self.entry = entry_class
        if tr:
            self.tr = tr
        else:
            random_slave = rs
            self.tr = TransactionAction(master_name, session_class_name, random_slave)

    @sql_wrapper(Query.all)
    def get_all(self, where_dict=None, **kwargs):
        session = self.tr.start()
        if where_dict:
            return session.query(self.entry).filter_by(**where_dict)
        return session.query(self.entry)

    def filter_fetch(self, query, limit=None, offset=None, page=None, per_page=None, order_by=None, count=None,
                     **kwargs):
        """
        :param query:
        :param limit:
        :param offset:
        :param page:
        :param per_page:
        :param order_by: 排序，示例[{"XX":"asc"},{"YY":"desc"}] 或者 {"XX":"asc"}
        :return:
        """
        if order_by:
            orders = []
            if type(order_by) == dict:
                for k, v in order_by.items():
                    if getattr(self.entry, k, None) is not None:
                        orders.append(getattr(getattr(self.entry, k), v)())
            elif type(order_by) == list:
                for item in order_by:
                    for k, v in item.items():
                        if getattr(self.entry, k, None) is not None:
                            orders.append(getattr(getattr(self.entry, k), v)())
            if orders:
                query = query.order_by(*orders)
        total = 0
        if page:
            total = query.count()
            query = query.offset((int(page) - 1) * int(per_page))
            query = query.limit(int(per_page))

        else:
            if limit:
                query = query.limit(limit)
            if offset:
                query = query.offset(offset)
        if count:
            if page:
                res = total
            else:
                res = query.count()
        else:
            res = query.all()
        if page:
            return res, total
        return res

    def simple_get_all_filter(self, params, condition_type='and', **kwargs):
        session = self.tr.start()
        if params:
            if type(params) == list:
                arg_list = []
                for one in params:
                    if isinstance(one[2], str) and not one[2].startswith("'") and not one[2].startswith("\""):
                        one[2] = "'{}'".format(one[2])
                    # print("self.entry.{}{}{}".format(one[0], one[1], one[2]))
                    arg_list.append(eval("self.entry.{}{}{}".format(one[0], one[1], one[2])))
                if condition_type == 'and':
                    condition_func = and_
                else:
                    condition_func = or_
                q = session.query(self.entry).filter(condition_func(*arg_list))
            elif type(params) == dict:
                filters = []
                for k, v in params.items():
                    if type(v) is list:
                        filters.append(getattr(self.entry, k).in_(v))
                    else:
                        filters.append(getattr(self.entry, k) == v)
                q = session.query(self.entry).filter(*filters)
            else:
                q = session.query(self.entry)
        else:
            q = session.query(self.entry)
        res = self.filter_fetch(q, **kwargs)
        self.tr.end()
        return res

    def simple_get_join(self, join_entry, on_dict, *args, order_by=None, **kwargs):
        session = self.tr.start()
        eval_str = 'and'.join(['{}.{}=={}.{}'.format(self.entry.__name__, x, join_entry.__name__, on_dict[x]) for x in
                               on_dict])
        temp = res = session.query(self.entry, join_entry).outerjoin(join_entry, eval(
            eval_str))
        if args:
            temp = temp.filter(*args)
        if order_by:
            orders = []
            for k, v in order_by.items():
                if getattr(self.entry, k, None) is not None:
                    orders.append(getattr(getattr(self.entry, k), v)())
                else:
                    orders.append(getattr(getattr(join_entry, k), v)())
            temp = temp.order_by(*orders)
        res = self.filter_fetch(temp, **kwargs)
        self.tr.end()
        return res

    def get_join_new(self, join_entry, on_dict, *args,
                     order_by=None, in_filter=None, equal_filter=None, range_filter=None, **kwargs):
        session = self.tr.start()
        eval_str = 'and'.join(['{}.{}=={}.{}'.format(self.entry.__name__, x, join_entry.__name__, on_dict[x]) for x in
                               on_dict])
        temp = res = session.query(self.entry, join_entry).outerjoin(join_entry, eval(eval_str))
        filters = []
        if equal_filter:
            for k, v in equal_filter.items():
                if v is not None:
                    if getattr(self.entry, k, None) is not None:
                        key = getattr(self.entry, k)
                    else:
                        key = getattr(join_entry, k)
                    filters.append(key == v)
        if in_filter:
            for k, v in in_filter.items():
                if v is not None:
                    if getattr(self.entry, k, None) is not None:
                        key = getattr(self.entry, k)
                    else:
                        key = getattr(join_entry, k)
                    filters.append(key.in_(v))
        if range_filter:
            for k, v in range_filter.items():
                if v:
                    if getattr(self.entry, k, None) is not None:
                        key = getattr(self.entry, k)
                    else:
                        key = getattr(join_entry, k)
                    if v[0] is not None:
                        filters.append(key >= v[0])
                    if v[1] is not None:
                        filters.append(key <= v[1])
        temp = temp.filter(*filters)
        if args:
            temp = temp.filter(*args)
        if order_by:
            orders = []
            for k, v in order_by.items():
                if getattr(self.entry, k, None) is not None:
                    orders.append(getattr(getattr(self.entry, k), v)())
                else:
                    orders.append(getattr(getattr(join_entry, k), v)())
            temp = temp.order_by(*orders)
        res = self.filter_fetch(temp, **kwargs)
        self.tr.end()
        return res

    def simple_get_all(self, extra_dict=None, **kwargs):
        """
        简单查询，
        :param extra_dict:
        :param kwargs:查询条件 都是逻辑与的关系
        :return: 
        """
        if not extra_dict:
            extra_dict = dict()
        or_list = kwargs.pop("or_list", None)
        session = self.tr.start()
        try:
            if kwargs:
                filters = []
                for k, v in kwargs.items():
                    if type(v) is list:
                        filters.append(getattr(self.entry, k).in_(v))
                    else:
                        filters.append(getattr(self.entry, k) == v)
                q = session.query(self.entry).filter(*filters)
            elif or_list:
                arg_list = []
                for one in or_list:
                    arg_list.append(eval("{}.{}=={}".format(self.entry.__name__, one[0], one[-1])))
                q = session.query(self.entry).filter(or_(*arg_list))
            else:
                q = session.query(self.entry)
            res = self.filter_fetch(q, **extra_dict)
            self.tr.end()
        except:
            traceback.print_exc()
            session.rollback()
            session.close()
            return
        return res

    def simple_get_one(self, first=False, **kwargs):
        """
        简单查询，
        :param first:是否取一个
        :param kwargs:查询条件 都是逻辑与的关系
        :return: 
        """
        session = self.tr.start()
        if kwargs:
            res = session.query(self.entry).filter_by(**kwargs).limit(1)
        else:
            res = session.query(self.entry).limit(1)
        if first:
            res = res.first()
        else:
            res = res.all()
        self.tr.end()
        return res

    def insert(self, **kwargs):
        session = self.tr.start()
        # print(id(session))
        try:
            obj = self.entry(**kwargs)
            session.add(obj)
            session.flush()
            # session.commit()
            self.tr.end()
            return obj
        except Exception as e:
            session.rollback()
            session.close()
            raise (e)

    def simple_delete_one(self, **kwargs):
        """
        简单查询删除，
        :param kwargs:查询条件 都是逻辑与的关系
        :return:
        """
        session = self.tr.start()
        if kwargs:
            res = session.query(self.entry).filter_by(**kwargs).first()
        else:
            res = session.query(self.entry).first()
        session.delete(res)
        self.tr.end()
        return res

    def simple_delete_all(self, **kwargs):
        """
        简单查询删除，
        :param kwargs:查询条件 都是逻辑与的关系
        :return:
        """
        session = self.tr.start()
        filters = []
        if kwargs:
            for k, v in kwargs.items():
                if type(v) is list:
                    filters.append(getattr(self.entry, k).in_(v))
                else:
                    filters.append(getattr(self.entry, k) == v)
            # res = self.entry.__table__.delete().where(*filters)
            res = session.query(self.entry).filter(*filters).delete(synchronize_session=False)
        else:
            res = session.query(self.entry).delete()
        self.tr.end()
        return res

    def batch_insert(self, data_list):
        session = self.tr.start()
        try:
            objs = [self.entry(**kwargs) for kwargs in data_list]
            session.add_all(objs)
            session.flush()
            res = objs
            self.tr.end()
            return res
        except Exception as e:
            session.rollback()
            session.close()
            raise (e)

    def batch_insert_ignore(self, data_list):  # 这个不支持transaction的装饰器事务
        session = self.tr.start()
        try:
            return_list = []
            for kwargs in data_list:
                obj = self.entry(**kwargs)
                try:
                    session.add(obj)
                    session.commit()
                    return_list.append(obj)
                except Exception as e:
                    if 'Duplicate entry' in str(e):
                        session.rollback()

                    else:
                        raise e
            # session.flush()
            self.tr.end()
            return return_list
        except Exception as e:
            session.rollback()
            session.close()
            raise (e)

    def batch_insert_update(self, data_list):
        if len(data_list) == 0:
            return False
        insert_keys = data_list[0].keys()
        sql = "INSERT INTO `" + self.entry.__tablename__ + "`("
        sql += ','.join(["`" + x + "`" for x in insert_keys])
        sql += ')VALUES'
        temp_list = []
        for x in data_list:
            temp_str = '('
            tt_list = []
            for k in insert_keys:
                if x[k] is None:
                    tt_list.append("null")
                elif type(x[k]) == str:
                    tt_list.append("'" + x[k] + "'")
                else:
                    tt_list.append(str(x[k]))
            temp_str += ','.join(tt_list)
            temp_str += ')'
            temp_list.append(temp_str)
        sql += ",".join(temp_list)
        sql += " ON DUPLICATE KEY UPDATE "
        temp_list = ["`" + x + "`=values(`" + x + "`)" for x in insert_keys]
        sql += ','.join(temp_list)
        self.execute(sql, _is_direct=True)
        return True

    def simple_update(self, update_query_condition={}, **kwargs):
        session = self.tr.start()
        try:
            res = session.query(self.entry).filter_by(**update_query_condition).update(kwargs)
            # session.flush()
            self.tr.end()
            return res > 0
        except Exception as e:
            session.rollback()
            session.close()
            raise (e)

    def simple_filter_update(self, filters=None, **kwargs):
        session = self.tr.start()
        try:
            res = session.query(self.entry).filter(*filters).update(kwargs, synchronize_session=False)
            # session.flush()
            self.tr.end()
            return res > 0
        except Exception as e:
            session.rollback()
            session.close()
            raise (e)

    def escape_sql(self, sql):
        sql = sql.replace(":", "\\:")
        return sql

    def execute(self, sql, *args, _is_direct=False, **kwargs):
        session = self.tr.start()
        # session.begin_nested()
        if _is_direct:
            sql = self.escape_sql(sql)
        res = session.execute(sql, *args, **kwargs)
        self.tr.end()
        return res

    def select_execute(self, sql, num=0, *args, _is_direct=False, **kwargs):
        session = self.tr.start()
        if _is_direct:
            sql = self.escape_sql(sql)
        res = session.execute(sql, *args, **kwargs).fetchall()
        if type(num) == int and num > 0:
            res = res[:num]
        elif type(num) == int and num == 0:
            pass
        else:
            res = None
        self.tr.end()
        return res

    @staticmethod
    def select(sql):
        def dec(func):
            def wrapper(self, *args, **kwargs):
                orm_keys = [x for x in self.entry.__dict__.keys() if not x.startswith('_')]
                f_sql = "select " + ','.join(orm_keys) + " from " + self.entry.__name__ + " where " + sql.format(
                    **kwargs)
                result = self.select_execute(f_sql)
                sql_result = []
                for one in result:
                    entry_dict = dict(zip(orm_keys, list(one)))
                    sql_result.append(self.entry(**entry_dict))
                return sql_result

            return wrapper

        return dec


class ObjectCovertToDictDao(BaseDao):

    def __deal_res(self, res):
        if type(res) == tuple:
            temp = [one.to_dict() for one in res[0]]
            res[0].clear()
            res[0].extend(temp)
            return res
        elif type(res) == list:
            return [one.to_dict() for one in res]
        return res

    def simple_get_all(self, **kwargs):
        res = super().simple_get_all(**kwargs)
        return self.__deal_res(res)

    def simple_get_all_filter(self, params, condition_type='and', **kwargs):
        res = super().simple_get_all_filter(params, condition_type=condition_type, **kwargs)
        return self.__deal_res(res)

    def simple_get_one(self, **kwargs):
        res = super().simple_get_all(**kwargs)
        return self.__deal_res(res)

    def simple_get_join(self, join_entry, on_dict, *args, order_by=None, **kwargs):
        res = super().simple_get_join(join_entry, on_dict, *args, order_by=order_by, **kwargs)
        return self.__deal_res(res)


if __name__ == "__main__":
    print(BaseDao.simple_get_all.__doc__)
    from pprint import pprint

    pprint({"code": 0, "msg": "success", "data": [
        {"a": 1, "b": "\u4e2d\u56fd\u94f6\u884c", "c": "1", "d": 0, "e": 1},
        {"a": 1, "b": "\u4e0a\u6d77\u94f6\u884c", "c": "1", "d": 0, "e": 2},
        {"a": 1, "b": "\u4e2d\u4fe1\u94f6\u884c", "c": "1", "d": 0, "e": 3}]})
    pass



# -*- coding: utf-8 -*-
"""
----------------------------------------------------
    File Name:              __init__.py
    Description
    Author:                 develop
    date:                   4/3/2019
----------------------------------------------------
"""
__author__ = 'develop'

import functools
import requests
import hashlib
import os
import yaml
import sys
import math
import datetime
import pytz
import base64
import socket
import re
import simplejson as json
from decimal import Context, ROUND_HALF_UP
from itertools import chain
from itertools import groupby
from collections import OrderedDict

PRIVATE_KEY = "MjIxLDM1"


class BaseError(Exception):
    base_msg = 'base error'

    def __init__(self, msg):
        self.msg = self.base_msg + ":" + msg

    def __str__(self):
        return repr(self.msg)


def window_func(func, seq, window_size):
    return list(chain(*map(func, window(seq, window_size))))


def window(seq, window_size):
    """
    :param seq:
    :param window_size:
    :return:
    """
    st = 0
    while st < len(seq):
        yield seq[st:st + window_size]
        st += window_size


def get_host_ip():
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(('8.8.8.8', 80))
        ip = s.getsockname()[0]
    finally:
        s.close()

    return ip


def get_platform():
    if "win" in sys.platform.lower():
        return "windows"
    elif "linux" in sys.platform.lower():
        return "linux"


def base64_encode_str(string):
    return base64.encodebytes(string.encode('utf-8')).decode('utf-8').strip()


def base64_decode_str(string):
    return base64.decodebytes(string.encode('utf-8')).decode('utf-8')


def module_import(module_name, sdk_name=None, version=None):
    try:
        __import__(module_name)
    except ImportError:
        os.system(
            'pip install {}'.format(sdk_name if sdk_name else module_name + ("" if not version else "==" + version)))
        __import__(module_name)


def encrypt(string, e_type='base64'):
    if e_type is None:
        return string
    elif e_type == 'base64':
        return base64_encode_str(string)
    else:
        raise Exception('e_type error!')


def decrypt(string, e_type='special'):
    if e_type is None or not string:
        return string
    elif e_type == 'base64':
        return base64_decode_str(string)
    elif e_type == 'special':
        if string.startswith('__c__'):
            return string[5:]
        return SpecialDecrypt.decrypt(string)
    else:
        raise Exception('e_type error!')


class SpecialDecrypt:
    base64str = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"

    @staticmethod
    def cipher_to_dec_array(cipher):
        arr = []
        base64_index_map = {}
        for i in range(len(SpecialDecrypt.base64str)):
            base64_index_map[SpecialDecrypt.base64str[i]] = i
        for c in cipher:
            arr.append(base64_index_map[c])
        return arr

    @staticmethod
    def dec_array_to_bin_string(arr):
        s = ''
        for i in arr:
            s += bin(i).replace('b', '').zfill(6)[-6:]
        return s

    @staticmethod
    def dealBinStringToBinArray(s):
        count = int(len(s) / 8)
        strArr = []
        for i in range(count):
            substr = s[:8]
            s = s[8:]
            strArr.append(substr)
        return strArr

    @staticmethod
    def binArrayToDecArray(binArray):
        decArray = []
        for b in binArray:
            decArray.append(int(b, 2))
        return decArray

    @staticmethod
    def rsaDecryt(cryptASCII, pow, divisor):
        return (int(cryptASCII) ** pow) % divisor

    @staticmethod
    def rsaArray(arr, privateKey):
        s = str(base64.decodebytes(privateKey.encode(encoding='utf-8')), 'utf-8')
        sps = s.split(',')
        for i in range(len(arr)):
            arr[i] = SpecialDecrypt.rsaDecryt(arr[i], int(sps[1]), int(sps[0]))
        return arr

    @staticmethod
    def decrypt(cipher, privateKey=PRIVATE_KEY):
        arr = SpecialDecrypt.cipher_to_dec_array(cipher)
        binaryString = SpecialDecrypt.dec_array_to_bin_string(arr)
        binArray = SpecialDecrypt.dealBinStringToBinArray(binaryString)
        enDecArray = SpecialDecrypt.binArrayToDecArray(binArray)
        rsaArr = SpecialDecrypt.rsaArray(enDecArray, privateKey)
        return ''.join(map(chr, rsaArr))


def read_parm(request):
    res = dict()
    res.update(request.args)
    res.update(request.form)

    if request.is_json and request.data:
        res.update(request.json)
    elif request.data != b'':
        res.update(json.loads(request.data.decode()))
    header_json = request.headers.get('json')
    if header_json:
        res.update(json.loads(header_json))
    return res


def construct_result(code=0, message="success", data=None, **kwargs):
    """
    构造结果
    :return: 
    """
    res = {"code": code, "msg": message, "data": data}

    for one in kwargs:
        if one not in res:
            res[one] = kwargs[one]
    return json.dumps(res, ensure_ascii=False), {"Content-type": "application/json"}


def post_req(url, **kwargs):
    error_message = ""
    count = 0
    while count < 3:
        try:
            r = requests.post(url, **kwargs)
            if r.status_code == 200:
                return True, r.json()
            else:
                error_message = "post_req {} error ,status_code:{},params:{}".format(url, r.status_code, kwargs)
                count += 1
        except Exception as e:
            error_message = "post_req {} error {},params:{}".format(url, e, kwargs)
            count += 1
        print(error_message)
    return False, error_message


def get_md5(file_name=None, bytes=None):
    every_catch_num = 32000
    if file_name:
        if not os.path.exists(file_name):
            raise Exception("文件不存在")
        m = hashlib.md5()
        with open(file_name, "rb") as fd:
            while True:
                x = fd.read(every_catch_num)
                if not x:
                    break
                m.update(x)
        return m.hexdigest()
    elif bytes:
        m = hashlib.md5()
        length = len(bytes)
        cur_pos = 0
        for one in range(int(length / every_catch_num)):
            x = bytes[cur_pos:cur_pos + every_catch_num]
            m.update(x)
            cur_pos += every_catch_num
        m.update(bytes[cur_pos:])
        return m.hexdigest()
    else:
        raise ValueError("没有获取md5的对象")


def read_yaml(yaml_file, encoding="utf-8"):
    config = dict()
    if not yaml_file:
        return config
    with open(yaml_file, encoding=encoding) as fd:
        config = yaml.safe_load(fd)
    return config


def get_roud_num(num, sep=2):
    # 导进来的这个包在对于大于1的数进行四舍五入的时候是没有问题的，但在小于1的数的时候就会出问题
    num = str(num)
    if sep < 0:
        return int(float(num))
    if float(num) >= 1:
        if sep != 0:
            a = ("%.{}f".format(sep)) % float(num)
        else:
            a = "%d." % int(float(num))
        c = Context(prec=(len(a) - 1), rounding=ROUND_HALF_UP).create_decimal(num)
        res = float(str(c))
    else:
        d = float(num) * math.pow(10, sep)
        print(d)
        if d - int(d) < 0.5:
            res = float(int(d) / math.pow(10, sep))
        else:
            i = (int(d) + 1) / math.pow(10, sep)
            res = float(i)
    if sep == 0:
        return int(res)
    return res


def mix_change(src_iterator, desc_iterator):
    """
    复合替换dict
    :param src_iterator:
    :param desc_iterator:
    :return:
    """
    if type(src_iterator) == dict:
        all_dict = dict(src_iterator, **desc_iterator)
        for key in all_dict.keys():
            if key not in src_iterator:
                src_iterator[key] = all_dict[key]
                continue
            value = src_iterator[key]
            if type(value) == dict:
                mix_change(value, desc_iterator.get(key, {}))
            elif type(value) == list:
                mix_change(value, desc_iterator.get(key, None))
            else:
                value = desc_iterator.get(key, value)
                src_iterator[key] = value
    elif type(src_iterator) == list:
        # 列表暂时也用替换
        if desc_iterator is not None:
            src_iterator.clear()
            src_iterator.extend(desc_iterator)
    else:
        raise Exception('config_iter invalid')


def replace_variables_collection(src, **variables_dict):
    if type(src) == list:
        for index, item in enumerate(src):
            if type(item) == list or type(item) == dict:
                replace_variables_collection(item, **variables_dict)
            else:
                src[index] = replace_variables(item, **variables_dict)
    elif type(src) == dict:
        for key, value in src.items():
            if type(value) == list or type(value) == dict:
                replace_variables_collection(value, **variables_dict)
            else:
                src[key] = replace_variables(value, **variables_dict)


def replace_variables(src, key, push_set=None, **variables_dict):
    """
    if src like 'something ${a:c} nothing!',return 'something {} nothing!'.format(variables_dict.get('a',c)
    :param src:
    :param variables_dict:
    :return:
    """
    if type(src) != str:
        return src
    if not push_set:
        push_set = set()
    re_compile = re.compile(r'(\$[{](.*?)[}])')  # 最小匹配
    match_obj = re.findall(re_compile, src)
    for replace_item, piece in match_obj:
        pieces = piece.split(":")
        replaced_key = pieces[0]
        if replaced_key == key:
            raise Exception("replaced_key {} cycle!!".format(replaced_key))
        # if replaced_key in push_set:
        #     raise Exception("replaced_key {} cycle!!".format(replaced_key))
        value = variables_dict.get(replaced_key)
        if not value:  # 没有找到对应
            if len(pieces) > 1:  # 没有找到对应，有默认值
                value = pieces[1]
            else:
                raise Exception("replaced_key {} can't be found!!".format(replaced_key))
        else:  # 找到对应
            match_obj = re.findall(re_compile, value)
            if match_obj:  # 嵌套参数替换
                push_set.add(key)
                value = replace_variables(value, replaced_key, push_set, **variables_dict)
        src = src.replace(replace_item, value)
        # push_set.s
    return src


def iter_change(src_iterator, name_dict, name_list=None, sep='_'):
    """
    扁平化替换
    Parameters
    ----------
    src_iterator
    name_dict
    name_list
    sep

    Returns
    -------

    """
    if name_list is None:
        name_list = []
    if type(src_iterator) == dict:
        for key, value in src_iterator.items():
            name_list.append(key)
            if type(value) == dict or type(value) == list:
                iter_change(value, name_dict, name_list, sep=sep)
            else:
                env_key = sep.join(name_list)
                src_iterator[key] = name_dict.get(env_key, value)
            name_list.pop(-1)
    elif type(src_iterator) == list:
        index = 0
        for one in src_iterator:
            name_list.append(str(index))
            if type(one) == dict or type(one) == list:
                iter_change(one, name_dict, name_list, sep=sep)
            else:
                env_key = sep.join(name_list)
                src_iterator[index] = name_dict.get(env_key, one)
            index += 1
            name_list.pop(-1)
    else:
        raise Exception('config_iter invalid')


temp = {
    "x": "xxxxxxxx",
    "y": "yyyyyyyyyy${x}",
    "z": "zzzzzz${a}zzzz${y}",
    "a": "aaaaaa${b}aaaaa${x}",
    "b": "bbbbbb${c}",
    "c": "ccccccc${d.0.f}${y}",
    "d": [
        {
            "e": "eeeeee${a},${b}",
            "f": "ffffff",
            "g": "gggggg",
        }
    ]
}

re_compile = re.compile(r'(\$[{](.*?)[}])')


def config_pre_deal(value, key):
    if type(value) != str:
        result = {"value": value, "relative_map": dict()}
    else:
        match_obj = re.findall(re_compile, value)
        if match_obj:
            # 嵌套
            replace_set = set()
            relative_map = dict()
            for replace_item, piece in match_obj:
                pieces = piece.split(":")
                replaced_key = pieces[0]
                if replaced_key == key:
                    raise Exception("replaced_key {} cycle!!".format(replaced_key))
                relative_map.setdefault(replaced_key, set()).add(replace_item)

            result = {"value": value, "relative_map": relative_map}
        else:
            result = {"value": value, "relative_map": dict()}
    return result


def flat_collect(config, pre="", flat_dict=None):
    if flat_dict is None:
        flat_dict = dict()
    if type(config) == dict:
        for key, value in config.items():
            if type(value) == dict or type(value) == list:
                flat_collect(value, pre=pre + key + ".", flat_dict=flat_dict)
            else:
                flat_dict[pre + key] = config_pre_deal(value, pre + key)

    elif type(config) == list:
        for index, item in enumerate(config):
            if type(item) == dict or type(item) == list:
                flat_collect(item, pre=pre + str(index) + ".", flat_dict=flat_dict)
            else:
                flat_dict[pre + str(index)] = config_pre_deal(item, pre + str(index))
    else:
        pass


def replace_flat_dict(key, value, relative_map, variables_dict, stack=None):
    if stack is None:
        stack = OrderedDict()
    if key in stack:
        raise Exception("replaced_key {} cycle!!".format(key))
    stack[key] = 1
    for replaced_key in relative_map:
        if replaced_key not in variables_dict:  # 該替換項目不存在
            for replace_item in relative_map[replaced_key]:
                if replace_item == "${" + replaced_key + "}":
                    raise Exception("replaced_key {} can't be found!!".format(replaced_key))
                elif replaced_key + ":" in replace_item:
                    value = value.replace(replace_item, replace_item[2:-1].split(":")[-1])
        else:
            for replace_item in relative_map[replaced_key]:
                value = value.replace(replace_item,
                                      str(replace_flat_dict(replaced_key, variables_dict[replaced_key]['value'],
                                                            variables_dict[replaced_key]['relative_map'],
                                                            variables_dict, stack)))
    variables_dict[key] = {
        "value": value,
        "relative_map": {}
    }
    stack.pop(key)
    return value


def get_filter_dic(dic, filter_keys=None, require_all=False):
    """

    :param dic:
    :param filter_keys:
    :param require_all:
    :return:
    字典按key过滤
    """
    if not filter_keys:
        return {}
    if not require_all:
        return {k: dic[k] for k in filter_keys if k in dic}
    else:
        return {k: dic[k] for k in filter_keys}


def get_changed_value(src, change_dict=None, not_exist_exception=None, callback=None):
    result = src
    if callback:
        result = callback(src)
    if change_dict:
        if result not in change_dict:
            if not_exist_exception:
                raise not_exist_exception
        result = change_dict.get(result, result)
    return result


def align_list(src_list, another_list, according_name):
    temp_dict = dict()
    for another_dict in another_list:
        temp_dict[another_dict[according_name]] = another_dict
    result_list = list()
    for src_dict in src_list:
        key = src_dict[according_name]
        if key not in temp_dict:
            # 没有找到对应的
            pass
        else:
            temp_list = [src_dict, temp_dict[key]]
            result_list.append(temp_list)
    return result_list


def align_list_union(src_list, another_list, according_name):
    temp_dict = dict()
    for another_dict in another_list:
        temp_dict[another_dict[according_name]] = another_dict
    temp_dict2 = dict()
    for src_dict in src_list:
        temp_dict2[src_dict[according_name]] = src_dict
    result_list = list()
    for src_dict in src_list:
        key = src_dict[according_name]
        if key not in temp_dict:
            # 没有找到对应的
            temp_list = [src_dict, dict.fromkeys(src_dict)]
            result_list.append(temp_list)
        else:
            temp_list = [src_dict, temp_dict[key]]
            result_list.append(temp_list)
    for ana_dict in another_list:
        key = ana_dict[according_name]
        if key not in temp_dict2:
            # 没有找到对应的
            temp_list = [dict.fromkeys(ana_dict), ana_dict]
            result_list.append(temp_list)
        # else:
        #     pass
        # temp_list = [ana_dict, temp_dict2[key]]
        # result_list.append(temp_list)
    return result_list


def dict_list_sum(dict_list, group_field, sum_field):
    """
    字典列表聚合求和
    按照group_field分组， 对sum_field求和
    :param dict_list: eg: [{'a': 1, 'b':3}, {'a': 1, 'b':2}, {'a': 2, 'b':2}, {'a': 2, 'b':10}]
    :param group_field: eg 'a'
    :param sum_field: eg 'b'
    :return: eg: {1: 5, 2: 12}
    """
    dic = {}
    for item in dict_list:
        group_value = item[group_field]
        if group_value not in dic:
            dic[group_value] = 0
        dic[group_value] += item[sum_field]
    return dic


def update_many_sql(data_list, mysql_table, merge_keys=None):
    query = ""
    if not merge_keys:
        merge_keys = []
    for data_dict in data_list:
        if not query:
            columns = ', '.join('`{0}`'.format(k) for k in data_dict)
            duplicates = ', '.join(
                '{0}=VALUES({0})'.format(k) if k not in merge_keys else '{0}={0}+VALUES({0})'.format(k) for k in
                data_dict)

            place_holders = ', '.join(':{}'.format(k) for k in data_dict)
            query = "INSERT INTO {0} ({1}) VALUES ({2})".format(mysql_table, columns, place_holders)
            query = "{0} ON DUPLICATE KEY UPDATE {1}".format(query, duplicates)
    return query


def update_many_sql_diff(data_list, mysql_table, diff_keys=None):
    query = ""
    if not diff_keys:
        diff_keys = []
    for data_dict in data_list:
        if not query:
            columns = ', '.join('`{0}`'.format(k) for k in data_dict)
            duplicates = ', '.join(
                '{0}=VALUES({0})'.format(k) if k not in diff_keys else '{0}=VALUES({0})-{0}'.format(k) for k in
                data_dict)

            place_holders = ', '.join(':{}'.format(k) for k in data_dict)
            query = "INSERT INTO {0} ({1}) VALUES ({2})".format(mysql_table, columns, place_holders)
            query = "{0} ON DUPLICATE KEY UPDATE {1}".format(query, duplicates)
    return query


def get_res(key, item_list):
    """

    :param key: 输入
    :param item_list: 匹配的对象
    :return:
    """
    index_res = set()
    for item in item_list:
        for i in range(len(item)):
            if key.upper() in item[i]:
                index_res.add(i)
                if len(index_res) == 10:
                    break
        if len(index_res) == 10:
            break
            # if item[i].strip().startswith(key):
            #     index_res.add(i)
            #     if len(index_res) == 5:
            #         break

    res = [[item_list[0][i], item_list[1][i], item_list[2][i]] for i in index_res]
    return res


def set_dict_list_values(dict_seq, key, value, **kwargs):
    from copy import deepcopy
    for dic in deepcopy(dict_seq):
        flag = True
        for k, v in kwargs.items():
            if dic[k] != v:
                flag = False
                break
        if flag:
            dic[key] = value
    return dict_seq


def dict_to_key(dic):
    key = ''
    for k, v in sorted(dic.items()):
        v = round(v, 4)
        key += "{}_{};".format(k, v)
    return key


def timestamp_2_date(timestamp, fmt="%Y-%m-%d %H:%M:%S"):
    return datetime.datetime.fromtimestamp(timestamp, tz=pytz.timezone('Asia/Shanghai')).strftime(fmt)


def date_range(start_date, end_date):
    start_date = datetime.datetime.strptime(start_date, '%Y-%m-%d')
    end_date = datetime.datetime.strptime(end_date, '%Y-%m-%d')
    for n in range(int((end_date - start_date).days) + 1):
        yield str((start_date + datetime.timedelta(n)).date())


def get_format_yield_list(org_date_list, org_yield_list, format_list):
    """

    :param org_date_list: 交易日日期列表
    :param org_yield_list: 交易日持仓金额列表
    :param format_list: 自然日日日期列表
    :return: 自然日持仓金额列表
    第一天已对齐；自然日日日期列表 包含 交易日日期列表
    """
    # 二分法
    # seq_dict = {}
    # diff_list = sorted(set(format_list) - set(org_date_list))
    # for date in diff_list:
    #     seq = bisect_left(org_date_list, date)
    #     seq_dict[seq] = 1 + seq_dict.setdefault(seq, 0)

    # 需要插入的位置字典
    seq_dict = {}
    i = 0
    for j in range(len(format_list)):
        if i < len(org_date_list) and format_list[j] == org_date_list[i]:
            i += 1
        else:
            seq_dict[i] = 1 + seq_dict.setdefault(i, 0)
    format_yield_list = []
    pre_seq = 0
    for seq in sorted(seq_dict.keys()):
        format_yield_list += org_yield_list[pre_seq:seq] + seq_dict[seq] * [org_yield_list[seq - 1]]
        pre_seq = seq
    format_yield_list += org_yield_list[pre_seq:]
    return format_yield_list


def doublewrap(f):
    '''
    a decorator decorator, allowing the decorator to be used as:
    @decorator(with, arguments, and=kwargs)
    or
    @decorator
    '''

    @functools.wraps(f)
    def new_dec(*args, **kwargs):
        if len(args) == 1 and len(kwargs) == 0 and callable(args[0]):
            # actual decorated function
            return f(args[0])
        else:
            # decorator arguments
            return lambda realf: f(realf, *args, **kwargs)

    return new_dec


# def chain(*args):
#     s = True
#     for e in args:
#         if s:
#             a = e
#             s = False
#         else:
#             a += e
#     return a

def get_sum_line(*args):
    dict_list = [{'date': k, 'value': v} for k, v in
                 chain(*[zip(el[0], el[1]) for el in args])]
    dict_list = sorted(dict_list, key=lambda x: x['date'])
    sum_list = [{'date': key, 'value': sum(d['value'] for d in group)} for key, group in
                groupby(dict_list, key=lambda d: d['date'])]
    date_list = [item['date'] for item in sum_list]
    amount_list = [item['value'] for item in sum_list]
    return date_list, amount_list


if __name__ == "__main__":
    pass

    print(chain((1, 3), (3, 2)))
    print(*chain({1: 3}, {3: 2}))
    # print({**{1:3,3:1},**{3:2}})
    # def a(*, b=1, c=3):
    #     print(b + c)
    print((1, 3) + (3, 2))

    # from collections import Iterator,Iterable
    # print(decrypt('QXBwXzQ5MTIwODgw'))
    # print(encrypt('sd61131707'))
    # print(update_many_sql([{'code':1}],'table1'))
    # print(list(date_range('2019-09-11','2019-9-29')))

    # print(set_dict_list_values(
    #     [{'asset_id': 3, 'holding_amount': 3000, 'holding_share': 8000, 'sub_asset_id': 6, 'security_id': '000003.MF'},
    #      {'asset_id': 2, 'holding_amount': 3000, 'holding_share': 8000, 'sub_asset_id': 6, 'security_id': '000003.MF'}],
    #     'holding_share', 0, asset_id=3))
    # print(get_platform())
    # print(get_roud_num(3.15, 1))
    #
    #
    # def my_round(t, n):
    #     i = abs(round(t * (10 ** (n + 1))))
    #     j = i % 10
    #     if j >= 5:
    #         m = (int(i / 10) + 1) * 10
    #     else:
    #         m = int(i / 10) * 10
    #     ret = m / (10 ** (n + 1))
    #     return ret if i >= 0 else -ret
    #
    #
    # print(my_round(-1.09, 1))
    # print(get_changed_value('hh', {'h': 2}, callback=lambda x: x[0]))
    # from decimal import Decimal



# -*- coding: utf-8 -*-
"""
------------------------------------------------------------------
    File Name:              base
    Description:            
    Author:                 hzy
    date:                   2019/8/24
------------------------------------------------------------------
    Change Activity:
                            2019/8/24
------------------------------------------------------------------
"""

__author__ = 'hzy'

import time
from flask_templates.common.utils import BaseError, construct_result
from flask_templates.var import ErrorCode, USER_ID, TOKEN_PARAM_NAME, CHARACTER_PARAM_NAME, VISITOR_ID
from flask_templates.common.class_holder import ClassHolder
import functools


class ServiceError(BaseError):
    base_msg = 'service error'


class ServiceForViewError(ServiceError):
    message = ''
    code = 0

    def __init__(self):
        super().__init__(msg=self.message)

    @property
    def resp(self):
        return construct_result(self.code, self.message)


class TokenLostError(ServiceForViewError):
    code = ErrorCode.TOKEN_LOST_ERROR
    message = TOKEN_PARAM_NAME + ' lost!'

    def __init__(self, **kwargs):
        self.message = self.message.format(**kwargs)
        super().__init__()


class AuthError(ServiceForViewError):
    code = ErrorCode.AUTH_ERROR
    message = 'auth error!info:{info}'

    def __init__(self, **kwargs):
        self.message = self.message.format(**kwargs)
        super().__init__()


class RangeError(ServiceForViewError):
    code = ErrorCode.RANGE_ERROR
    message = '{param} not in range {range_list}!'

    def __init__(self, **kwargs):
        self.message = self.message.format(**kwargs)
        super().__init__()


class ExceedLimitError(ServiceForViewError):
    code = ErrorCode.EXCEED_LIMIT_ERROR
    message = '{param}超过限制'

    def __init__(self, **kwargs):
        self.message = self.message.format(**kwargs)
        super().__init__()


class PermissionForbiddenError(ServiceForViewError):
    code = ErrorCode.PERMISSION_FORBIDDEN_ERROR
    message = '行内资产无法修改和删除'


class NotFoundError(ServiceForViewError):
    code = ErrorCode.NOT_FOUND_ERROR
    message = '{param}不存在'

    def __init__(self, **kwargs):
        self.message = self.message.format(**kwargs)
        super().__init__()


class DuplicateError(ServiceForViewError):
    code = ErrorCode.DUPLICATE_ERROR
    message = '{param}重复'

    def __init__(self, **kwargs):
        self.message = self.message.format(**kwargs)
        super().__init__()


class AccountError(ServiceForViewError):
    code = ErrorCode.ACCOUNT_ERROR
    message = '{msg}'

    def __init__(self, **kwargs):
        self.message = self.message.format(**kwargs)
        super().__init__()


class DBError(ServiceForViewError):
    code = ErrorCode.DB_ERROR
    message = '存储数据异常'

    def __init__(self, **kwargs):
        self.message = self.message.format(**kwargs)
        super().__init__()


class CountiFeignError(ServiceError):
    code = ErrorCode.COUNTI_FEIGN_ERROR
    message = 'counti request error info :{info}'

    def __init__(self, **kwargs):
        self.message = self.message.format(**kwargs)
        super().__init__(self.message)


class BankFeignError(ServiceError):
    code = ErrorCode.BANK_FEIGN_ERROR
    message = 'bank request error info :{info}'

    def __init__(self, **kwargs):
        self.message = self.message.format(**kwargs)
        super().__init__(self.message)


class CountiValidationError(ServiceError):
    code = ErrorCode.COUNTI_VALIDATION_ERROR
    def __init__(self, msg, data):
        self.message = msg
        self.messages =msg
        self.valid_data = data
        super().__init__(msg=self.message)


class BaseChecker:

    def check_access(self,*args,**kwargs):
        pass

def check_auth(character=None):
    """
    检验调用权限函数
    :param character:
    :return:
    """

    def auth(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            if TOKEN_PARAM_NAME not in kwargs:
                raise TokenLostError()
            character_name = kwargs.setdefault(CHARACTER_PARAM_NAME, None)
            result = ClassHolder().get(BaseChecker).check_access(kwargs[TOKEN_PARAM_NAME], character_name)
            # kwargs.pop(TOKEN_PARAM_NAME)
            if character_name is None:
                kwargs[USER_ID] = result
            kwargs[VISITOR_ID] = result
            return func(*args, **kwargs)

        return wrapper

    return auth


def get_user_id(kwargs):
    token = kwargs[USER_ID]

    return token['user_id']


def catch_feign(service_error):
    def do(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            # res = func(*args, **kwargs)
            try:
                res = func(*args, **kwargs)
            except Exception as e:
                raise service_error(info=str(e))
            return res

        return wrapper

    return do




# coding: utf-8
from sqlalchemy import *
from sqlalchemy.dialects.mysql import *
from sqlalchemy.ext.declarative import declarative_base
import datetime
from decimal import Decimal

DeclarativeBase = declarative_base()
metadata = DeclarativeBase.metadata


class Base(DeclarativeBase):
    __abstract__ = True

    def to_dict(self):
        _dict = self.__dict__
        if "_sa_instance_state" in _dict:
            del _dict["_sa_instance_state"]
        for key, value in _dict.items():
            if isinstance(value, datetime.datetime):
                _dict[key] = str(value)
            elif isinstance(value, Decimal):
                _dict[key] = float(value)
        return _dict


class Example(Base):
    __tablename__ = 'asset_info'

    asset_id = Column(TINYINT(2), primary_key=True, comment='资产类型id')
    asset_type = Column(String(20), nullable=False, unique=True, comment='资产类型')
    alias = Column(String(20), nullable=False, unique=True, comment='类型别名')
    description = Column(String(50), comment='详细描述')
    create_time = Column(TIMESTAMP, nullable=False, server_default=text("CURRENT_TIMESTAMP"), comment='创建时间')
    update_time = Column(TIMESTAMP, nullable=False,
                         server_default=text("CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP"), comment='更新时间')




# -*- coding: utf-8 -*-
"""
------------------------------------------------------------------
    File Name:              __init__.py
    Description:            
    Author:                 hzy
    date:                   2020/11/17
------------------------------------------------------------------
    Change Activity:
                            2020/11/17
------------------------------------------------------------------
"""
__author__ = 'hzy'

from flask import Flask
from flask_cors import CORS
from flask_templates.configs import config_update, BEFORE_CONFIG, BASE_CONFIG, template, nacos, before_config_filename, \
    config_replace
from flask_templates.var import API_PREFIX_PATH, POINT_PREFIX_PATH
from flask_templates.common.logger import logger
import time


class FlaskTemplate:

    def __init__(self, resource_path='project/resource', setting={}, *args, **kwargs):
        self.fresh_scope(resource_path=resource_path, setting=setting)
        self.app = self.create_app(*args, **kwargs)
        self.web_config = template.WEB_CONFIG

    @staticmethod
    def fresh_scope(resource_path='project/resource', setting={}):
        before_config = config_update(src_config=BEFORE_CONFIG, resource_path=resource_path,
                                      call_fix=[nacos.config_fix], src_config_filename=before_config_filename)
        config_replace(before_config)
        config_update(resource_path=resource_path, setting=before_config, final_setting=setting,
                      call_fix=[template.config_fix])

    @staticmethod
    def fresh_scope_final():
        config_replace(BASE_CONFIG)

    @staticmethod
    def create_app(*args, **kwargs):
        app = Flask(__name__)
        # init()
        from flask_templates.views import api as api_blueprint
        app.register_blueprint(api_blueprint, url_prefix=API_PREFIX_PATH)

        from flask_templates.views import point as point_blueprint
        app.register_blueprint(point_blueprint, url_prefix=POINT_PREFIX_PATH)

        CORS(app, supports_credentials=True)
        app.logger = logger
        return app

    def run(self, **kwargs):
        app_run_params = self.web_config.copy()
        app_run_params.update(kwargs)
        self.app.run(host=app_run_params.pop('host', None), port=app_run_params.pop('port', None),
                     debug=app_run_params.pop('debug', None),
                     load_dotenv=app_run_params.pop('load_dotenv', True))


if __name__ == "__main__":
    start_t = time.time()

    pass

    print("use time: %s" % (time.time() - start_t))



from flask_templates import FlaskTemplate
from flask_templates.common.logger import logger
from flask_templates.common.class_holder import ClassHolder
from flask_templates.nacosinstance import NacosClient
from flask_templates.configs import BASE_CONFIG

FlaskTemplate.fresh_scope()
ClassHolder().put(NacosClient(only_reader=True))



def sum(a,b):
    logger.info(BASE_CONFIG.get("db-master-port"))
    return a+b


# -*- coding: utf-8 -*-
"""
----------------------------------------------------
    File Name:              base
    Description             
    Author:                 hzy
    date:                   2019/4/4
----------------------------------------------------
"""


__author__ = 'hzy'

import functools
import time
from flask import request
import traceback

from flask_templates.common.utils import read_parm, construct_result, BaseError, doublewrap
from flask_templates.var import ErrorCode, INNER_ERROR_STRING
from flask_templates.service.base import *
from flask_templates.common.logger import logger

class ViewError(BaseError):
    base_msg = 'view error'
    message = ''
    code = 0

    def __init__(self):
        super().__init__(msg=self.message)

    @property
    def resp(self):
        return construct_result(self.code, self.message)


class ParamLostError(ViewError):
    code = ErrorCode.PARAM_LOST_ERROR
    message = 'param {param} lost!'

    def __init__(self, **kwargs):
        self.message = self.message.format(**kwargs)
        super().__init__()


class ParamIncorrectError(ViewError):
    code = ErrorCode.PARAM_INCORRECT_ERROR
    message = 'param {param} incorrect!'

    def __init__(self, **kwargs):
        self.message = self.message.format(**kwargs)
        super().__init__()


class ViewValidationError(ViewError):
    code = ErrorCode.VALIDATION_ERROR

    def __init__(self, msg, data):
        self.message = msg
        self.messages =msg
        self.valid_data = data
        super().__init__()


def check_param_range(param_name, param, _min=None, _max=None):
    try:
        if _min is None:
            condition = param <= _max
        elif _max is None:
            condition = param >= _min
        else:
            condition = _min <= param <= _max
        if condition:
            return
        else:
            raise ParamIncorrectError(param=param_name)
    except:
        raise ParamIncorrectError(param=param_name)




@doublewrap
# def check_request_params(*deco_params,serialize_cls=None):
def check_request_params(func, *deco_params,serialize_cls=None, auth=True, construct=True):
    @functools.wraps(func)
    def wrap(*args, **kwargs):
        # print(deco_params)
        try:
            params = read_parm(request)
            for one in deco_params:
                if one not in params:
                    raise ParamLostError(param=one)
                kwargs[one] = params[one]
            for one in params:
                if one not in kwargs:
                    kwargs[one] = params[one]
            if serialize_cls is not None:
                kwargs = serialize_cls().load(kwargs)
            service_func = func
            if auth:
                service_func = check_auth()(func)
            result = service_func(*args, **kwargs)
            extra_dict = dict()
            if isinstance(result, tuple):
                extra_dict.update(result[1])
                result = result[0]
            return construct_result(data=result, **extra_dict) if construct else result
        except Exception as e:
            method = request.method
            url = request.url
            logger.error('{} {} {}'.format(method, url, kwargs))
            logger.error(traceback.format_exc())
            if isinstance(e, (ViewError, ServiceForViewError)):
                return e.resp
            if isinstance(e,BankFeignError):
                return construct_result(ErrorCode.BANK_FEIGN_ERROR, message=INNER_ERROR_STRING)
            if isinstance(e, CountiFeignError):
                return construct_result(ErrorCode.COUNTI_FEIGN_ERROR, message=INNER_ERROR_STRING)
            if isinstance(e, CountiValidationError):
                return construct_result(ErrorCode.COUNTI_VALIDATION_ERROR, message=INNER_ERROR_STRING)
            return construct_result(ErrorCode.SYSTEM_BUG_ERROR, message=INNER_ERROR_STRING)

    return wrap

    # return wrapper


if __name__ == "__main__":
    start_t = time.time()

    pass

    print("use time: %s" % (time.time() - start_t))



# -*- coding: utf-8 -*-
"""
----------------------------------------------------
    File Name:              __init__.py
    Description
    Author:                 develop
    date:                   4/3/2019
----------------------------------------------------
"""
__author__ = 'develop'

import random
from hashlib import md5
from threading import Lock
from copy import deepcopy
from contextlib import closing
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from sqlalchemy.ext.declarative import declarative_base
from flask_templates.configs.template import DB_CONFIG, MYSQL_CONFIG
from flask_templates.common.utils import decrypt

Base = declarative_base()

_MANAGER_LOCKS = Lock()

common_config = {
    'pool_recycle': 30,  # 数据库连接超时回收时间
    'echo': MYSQL_CONFIG.get("echo", False),
}


class MySQL(object):
    """数据库连接类"""
    INSTANCE = {}

    def __new__(cls, master_name=None, charset='utf8'):
        """
        构造方法
        :param master_name: 主库名
        :param charset: 数据库字符集
        :return:
        """
        e_type = 'special' if DB_CONFIG.get('encryption') else None
        slave_config = []
        master_config = MYSQL_CONFIG.get("master", {})
        use_slave = MYSQL_CONFIG.get('use_slave', False)
        if use_slave:
            slave_config = MYSQL_CONFIG.get('slaves', [])

        config_hash = md5(("".join(map(lambda i: "{}{}".format(i[0], i[1]), sorted(master_config.items()))) +
                           str(use_slave)).encode()).hexdigest()
        if config_hash in cls.INSTANCE:
            return cls.INSTANCE[config_hash]

        with _MANAGER_LOCKS:
            if config_hash not in cls.INSTANCE:
                instance = object.__new__(cls)
                instance.engine = cls.get_engine(master_config, charset, e_type)
                slave_session_class_dict = {}
                cls.INSTANCE[config_hash] = instance
                if use_slave:
                    for index, one_slave in enumerate(slave_config):
                        name = one_slave.get('name', str(index))
                        if name in slave_session_class_dict:
                            if type(slave_session_class_dict[name]) != list:
                                slave_session_class_dict[name] = [slave_session_class_dict[name]]
                            cur_engine = cls.get_engine(one_slave, charset, e_type)
                            slave_session_class_dict[name].append(cls.get_session_class(cur_engine))
                        else:
                            cur_engine = cls.get_engine(one_slave, charset, e_type)
                            slave_session_class_dict[name] = cls.get_session_class(cur_engine)
                instance.slave_session_class_dict = slave_session_class_dict
        return cls.INSTANCE[config_hash]

    @staticmethod
    def get_engine(config, charset, e_type):
        """
        创建sqlalchemy数据库引擎
        :param config: 数据库配置字典
        :param charset: 数据库字符集
        :param e_type: 是否加密密码
        :return:
        """
        common_db_config = deepcopy(common_config)
        config = deepcopy(config)
        config.update({
            'password': decrypt(config["password"], e_type),
            'charset': charset
        })
        if 'name' in config:
            config.pop('name')
        common_db_config.update(config)
        order = ['user', 'password', 'host', 'port', 'database', 'charset']
        return create_engine(
            "mysql+pymysql://{}:{}@{}:{}/{}?charset={}".format(*[common_db_config.pop(i) for i in order]),
            echo_pool=MYSQL_CONFIG.get("echo", False), **common_db_config
        )

    @staticmethod
    def get_session_class(cur_engine):
        return sessionmaker(bind=cur_engine, autoflush=False, autocommit=False, expire_on_commit=False)

    def sql_session(self, session_class_name=None, random_slave=False):
        db_session = sessionmaker(bind=self.engine, autoflush=False, autocommit=False, expire_on_commit=False)
        if random_slave:
            if len(self.slave_session_class_dict) == 0:
                session_class = db_session
            else:
                key = random.choice(list(self.slave_session_class_dict.keys()))
                key = session_class_name if session_class_name in self.slave_session_class_dict else key
                session_class = self.slave_session_class_dict[key] if type(
                    self.slave_session_class_dict[key]) != list else random.choice(self.slave_session_class_dict[key])
        elif session_class_name:
            if session_class_name in self.slave_session_class_dict:
                session_class = self.slave_session_class_dict[session_class_name]
            else:
                raise Exception("salve name {} don't exist".format(session_class_name))
        else:
            session_class = db_session
        return session_class()


# 初始化SQLalchemy引擎,其中strategy='threadlocal' 是使用本地线程模式。
engine = MySQL().engine
DB_Session = sessionmaker(bind=engine, autoflush=False, autocommit=False, expire_on_commit=False)


def commit_connection(func):
    """Single Mysql commit Connection Manager"""

    def args_wraps(*args, **kwargs):
        with closing(engine.raw_connection()) as con:
            ret = func(cursor=con.cursor(), *args, **kwargs)
            con.commit()
            return ret

    return args_wraps


if __name__ == "__main__":
    print(DB_Session().execute("select now()").fetchall())



# -*- coding: utf-8 -*-
"""
------------------------------------------------------------------
    File Name:              logger
    Description:            
    Author:                 hzy
    date:                   2019/8/23
------------------------------------------------------------------
    Change Activity:
                            2019/8/23
------------------------------------------------------------------
"""
__author__ = 'hzy'

import time

import logging
import logging.handlers
from flask_templates.configs.template import LOG_CONFIG


def get_logger(logger_name, logger_level=logging.INFO):
    logger = logging.getLogger(logger_name)
    logger.setLevel(logger_level)

    log_format = "%(name)s\t%(asctime)s\t%(pathname)s\t[line:%(lineno)d]\t %(levelname)s\t %(message)s"
    formater = logging.Formatter(log_format)

    # 存入文件的日志
    # handler = logging.handlers.TimedRotatingFileHandler(logger_location, "midnight", 1, days, encoding="utf-8")
    # handler.suffix = "%Y%m%d"
    # handler.setFormatter(formater)
    # logger.addHandler(handler)

    if LOG_CONFIG.get('enable_mail'):
        # 发邮件的日志
        mail_config = LOG_CONFIG.get('mail')
        sh = logging.handlers.SMTPHandler(mail_config['smtp_server'], mail_config['sender'], mail_config['receivers'],
                                          mail_config['subject'],
                                          credentials=(mail_config['user'], mail_config['password']), secure=())
        sh.setLevel(logging.ERROR)
        sh.setFormatter(formater)
        logger.addHandler(sh)

    return logger


logger = None
default_level = getattr(logging, LOG_CONFIG.get('level', "INFO"), logging.INFO)
try:
    from hzylog import logger_helper

    logger = logger_helper.get_logger(name='hzy', webhook=True)
except ModuleNotFoundError as e:

    logging.basicConfig(level=default_level,
                        format='%(asctime)s - %(name)s - %(filename)s - %(funcName)s - %(lineno)d - %(levelname)s - %(message)s')
    # logger = logging.getLogger(__name__)
    logger = get_logger(__name__, logger_level=default_level)

    logger.warning("module %s can't import!", 'hzylog')


class PreLogger:
    logger_dict = {}


def apply_logger(logger_name=None):
    global logger
    logger_dict = logging.Logger.manager.loggerDict
    if logger_name:
        if logger_dict.get(logger_name):
            PreLogger.logger_dict.setdefault(logger_name, []).append(logger_dict.get(logger_name))
        logger_dict[logger_name] = logger
    else:
        for x in logger_dict:
            PreLogger.logger_dict.setdefault(x, []).append(logger_dict[x])
            logger_dict[x] = logger


def reset_logger(logger_name=None):
    logger_dict = logging.Logger.manager.loggerDict
    if logger_name:
        if PreLogger.logger_dict.get(logger_name):
            logger_dict[logger_name] = PreLogger.logger_dict.get(logger_name).pop(-1)
    else:
        for x in PreLogger.logger_dict:
            if PreLogger.logger_dict.get(x):
                logger_dict[x] = PreLogger.logger_dict.get(x).pop(-1)


if __name__ == "__main__":
    start_t = time.time()
    logger.error('123')

    pass

    print("use time: %s" % (time.time() - start_t))



import time

from setuptools import setup, find_packages

setup(
    name='flask-templates',
    version='0.0.47',
    author='hzy',
    platforms='any',
    packages=find_packages(),
    package_data={'flask_templates': ['resource/*.yaml', '*.py', '*.pyd'],
                  '': ['*.yaml']
                  },
    install_requires=[
        'flask==1.1.1',
        'requests==2.22.0',
        'pyyaml==5.1.2',
        'pymysql==0.9.3',
        'sqlacodegen==2.1.0',
        'pytest-cov',
        'tenacity==5.1.1',
        'flask-cors==3.0.8',
        'redis==3.3.8',
        'simplejson==3.16.0',
        'marshmallow==3.2.1',
        'pytz==2020.4',
        'flask_apscheduler==1.11.0',
        'nacos-sdk-python==0.1.5',
        'celery==4.4.7',
        'flower==0.9.7'
    ]
)



# -*- coding: utf-8 -*-
"""
------------------------------------------------------------------
    File Name:              var
    Description:            
    Author:                 hzy
    date:                   2019/8/23
------------------------------------------------------------------
    Change Activity:
                            2019/8/23
------------------------------------------------------------------
"""
__author__ = 'hzy'

import time
import re


class Permission:
    VIEW_DATA = 2 ** 0
    VIEW_DATA_PAD = 2 ** 1

    WRITE_ARTICLE = 2 ** 2

    PUBLISH_DRAFT_ARTICLE = 2 ** 3
    PUBLISH_WITHDRAWN_ARTICLE = 2 ** 4

    EDIT_PUBLISHED_ARTICLE = 2 ** 5
    EDIT_DRAFT_ARTICLE = 2 ** 6
    EDIT_WITHDRAWN_ARTICLE = 2 ** 7
    EDIT_TO_BE_PUBLISHED_ARTICLE = 2 ** 8

    WITHDRAWN_PUBLISHED_ARTICLE = 2 ** 9
    WITHDRAWN_TO_BE_PUBLISHED_ARTICLE = 2 ** 10

    DELETE_PUBLISHED_ARTICLE = 2 ** 11
    DELETE_TO_BE_PUBLISHED_ARTICLE = 2 ** 12
    DELETE_DRAFT_ARTICLE = 2 ** 13
    DELETE_WITHDRAWN_ARTICLE = 2 ** 14

    RECOMMEND_ARTICLE = 2 ** 15

    ADD_USER = 2 ** 16
    DELETE_USER = 2 ** 17
    EDIT_USER = 2 ** 18
    # RESET_USER_PASSWORD = 2 ** 19

    ADD_ROLE = 2 ** 19
    UPDATE_ROLE_PERMISSION = 2 ** 20
    DELETE_ROLE = 2 ** 21

    # USER_STATISTIC = 2 ** 23  # 后台首页用户数据统计
    # ARTICLE_STATISTIC = 2 ** 24  # 后台首页文章统计
    # ARTICLE_WRITE_RANK = 2 ** 25  # 后台首页文章编写排行榜
    # ARTICLE_PUBLISH_RANK = 2 ** 26  # 后台首页文章发表排行榜
    # USER_PUBLISH_ARTICLE = 2 ** 27  # 后台首页用户发表文章
    # USER_WITHDRAWN_ARTICLE = 2 ** 28  # 后台首页用户撤回文章
    # USER_WRITE_ARTICLE = 2 ** 29  # 后台首页用户撰写文章

class ErrorCode:
    SYSTEM_BUG_ERROR = -500
    PARAM_INCORRECT_ERROR = -405
    PARAM_LOST_ERROR = -404
    TOKEN_LOST_ERROR = -403
    NOT_FOUND_ERROR = -407
    DUPLICATE_ERROR = -409
    AUTH_ERROR = -406
    RANGE_ERROR = -400
    VALIDATION_ERROR = -420
    COUNTI_FEIGN_ERROR = -501
    BANK_FEIGN_ERROR = -502
    EXCEED_LIMIT_ERROR = -411
    PERMISSION_FORBIDDEN_ERROR = -410
    DB_ERROR = -408
    ACCOUNT_ERROR = -412
    COUNTI_VALIDATION_ERROR = -499


# 描述
INNER_ERROR_STRING = 'inner_error 💀'

# blueprint prefix_path
API_PREFIX_PATH = '/api'
POINT_PREFIX_PATH = '/point'

# param
TOKEN_PARAM_NAME = 'token'
USER_ID = 'user_id'
CHARACTER_PARAM_NAME = 'character'
VISITOR_ID = 'visitor_id'

# date format
DEFAULT_YEAR_INTERVAL = 365
DATE_FORMAT = '%Y-%m-%d'
DATETIME_FORMAT = '%Y-%m-%d %H:%M:%S'

# cache prefix
ANALYSIS_RESULT = "analysis_result"
ANALYSIS_SRC = "analysis_src"
RISK_KEY = 'risk'
EF_LINES = 'ef_lines'
BENCHMARK_INFO = 'benchmark_info'
ASSET_INFO = 'asset_info'
PRODUCT_INFO = 'product_info'
PRE_ASSET_SECURITY = 'pre_asset_security'
PRE_AVAILABLE_ASSET_SECURITY = 'pre_available_asset_security'
RISK_PREDICT = 'risk_predict'
ASSET_PRICE_LATEST = 'asset_price_latest'
MC_PREDICT_RETURNS = 'mc_predict_returns'
CODE_SCORE = 'code_score'
CODE_INFO = 'code_info'
CODE_PERFORMANCE = 'code_performance'
CODE_RISK = 'code_risk'
CODE_TREND = 'code_trend'
CODE_QUOTE = 'code_quote'
CODE_PENETRATE = 'code_penetrate'
CODE_INDICATOR = 'code_indicator'
PORTFOLIO_TEMPLATES = 'portfolio_templates'
PORTFOLIOS_INDICATOR = 'portfolios_indicator'
CODE_TAG = 'code_tag'
CODE_COMPONENT = 'code_component'
CODE_TRANSFORM = 'code_transform'
FUND_INFO = 'fund_info'
# 前端字段映射
# 埋点映射
BURYING_POINT_DICT = {
    "service_id": 'sid',
    "page_id": 'pid',
    "stay_time": 'stm',
    "enter_time": 'etm',
    "addition": 'ad',
    "operation": 'o',
    "operation_action": 'oat',
    "operation_target": 'ota',
    "operation_time": 'otm',
    "operation_addition": 'oad'
}


if __name__ == "__main__":
    start_t = time.time()

    print("use time: %s" % (time.time() - start_t))



# -*- coding: utf-8 -*-
"""
----------------------------------------------------
    File Name:              redis_module
    Description             
    Author:                 develop
    date:                   2019/4/8
----------------------------------------------------
"""
__author__ = 'develop'

from hashlib import md5
from threading import Lock
from redis import Redis, ConnectionPool, sentinel
from copy import deepcopy
from functools import wraps
from flask_templates.common.utils import decrypt
from flask_templates.configs.template import REDIS_CONFIG
from flask_templates.common.logger import logger

_MANAGER_LOCKS = Lock()


def cache_decorator(redis_name=None, master=True):
    def wrapper(func):
        @wraps(func)
        def dec(*args, **kwargs):
            if "redis_client" not in kwargs and all([not isinstance(x, Redis) for x in args]):
                if "redis_name" in kwargs and not redis_name:
                    my_redis_name = kwargs['redis_name']
                else:
                    my_redis_name = redis_name
                kwargs['redis_client'] = MyRedis(redis_name=my_redis_name).get_redis_client(master=master)
            try:
                res = func(*args, **kwargs)
                return res
            except Exception as e:
                # import traceback
                # traceback.print_exc()
                logger.warning(str(e))

        return dec

    return wrapper


class MyRedis(object):
    """Redis连接类"""

    INSTANCE = {}

    def __new__(cls, redis_name=None):
        """
        singleton redis constructor
        :param redis_name: redis_name string in application.yaml
        :return:
        """
        e_type = 'special' if REDIS_CONFIG.get('encryption') else None
        if not redis_name:
            redis_config = deepcopy(REDIS_CONFIG.get('default'))
        else:
            redis_config = deepcopy(REDIS_CONFIG.get(redis_name))
            if not redis_config:
                raise Exception("no such redis configs named `{}`".format(redis_name))
        config_hash = md5("".join(map(
            lambda i: "{}{}".format(i[0], i[1]), sorted(redis_config.items())
        )).encode()).hexdigest()
        if config_hash in cls.INSTANCE:
            return cls.INSTANCE[config_hash]
        with _MANAGER_LOCKS:
            if config_hash not in cls.INSTANCE:
                instance = object.__new__(cls)
                redis_config['host'] = [x.strip() for x in redis_config['host'].split(',')]
                redis_config['port'] = [x.strip() for x in str(redis_config['port']).split(',')]
                redis_config['password'] = decrypt(redis_config['password'], e_type)
                instance.redis_name = redis_name
                instance.service_name = redis_config['service_name']
                instance.use_sentinel = redis_config.pop('use_sentinel') if 'use_sentinel' in redis_config else False
                if not instance.use_sentinel:
                    redis_config['host'] = redis_config['host'][0]
                    redis_config['port'] = redis_config['port'][0]
                instance.redis_config = redis_config
                MyRedis.construct(instance)
                cls.INSTANCE[config_hash] = instance
        return cls.INSTANCE[config_hash]

    @staticmethod
    def deal_with_hosts(hosts, ports):
        res = []
        for i, x in enumerate(hosts):
            x_host_piece = x.split(":")
            host = x_host_piece[0]
            port = ports[i]
            if len(x_host_piece) > 1:
                port = x_host_piece[1]
            res.append((host, port))
        return res

    @staticmethod
    def construct(instance):
        redis_config = instance.redis_config.copy()
        redis_name = instance.redis_name
        use_sentinel = instance.use_sentinel
        if use_sentinel:
            st = sentinel.Sentinel(MyRedis.deal_with_hosts(redis_config.pop('host'), redis_config.pop('port')))
            service_name = redis_config.pop('service_name')
            instance.master = st.master_for(service_name, **redis_config)
            instance.slave = st.slave_for(service_name, **redis_config)
        else:
            redis_config.pop("service_name", None)
            pool = ConnectionPool(**redis_config)  # decode_responses=True保证解析出来的不是bytes类型
            instance.master = instance.slave = Redis(connection_pool=pool)

    def refresh(self):
        MyRedis.construct(self)

    def get_redis_client(self, master=True):
        """返回一个Redis对象的实例"""
        if master:
            return self.master
        return self.slave


if __name__ == '__main__':
    myredis = MyRedis('default')
    print(id(myredis.get_redis_client()))
    a=myredis.get_redis_client()
    a.set("a",1)
    myredis.refresh()
    print(id(myredis.get_redis_client()))
    print(id(myredis.get_redis_client()))
    print(myredis.get_redis_client(False).get("a"))


# -*- coding: utf-8 -*-
"""
------------------------------------------------------------------
    File Name:              example
    Description:            
    Author:                 hzy
    date:                   2019/8/24
------------------------------------------------------------------
    Change Activity:
                            2019/8/24
------------------------------------------------------------------
"""
__author__ = 'hzy'

import time
from flask_templates.service.base import check_auth
from flask_templates.var import VISITOR_ID


# @check_auth()
def index(**kwargs):
    # da=WealthArticleDao()
    # res = da.insert(title='1233',title_image='12da',content='dasdad')
    # res2 = da.insert(title='12333', title_image='12da', content='dasdad')
    # res2 = WealthArticleDao().insert(title='1233',title_image='12da',content='dasdad')
    return f'example {kwargs[VISITOR_ID]}:hello,world!'


